<?php
 /**
 * i-doIT
 * 
 * LDAP-Module
 *
 * @package i-doit
 * @subpackage Modules
 * @author Dennis Stcken <dstuecken@i-doit.org>
 * @version 0.9
 * @copyright Copyright 2004-2008 - synetics GmbH
 * @license http://dev.perl.org/perl6/rfc/346.html Artistic License 2.0
 * 
 */
 define("C__LDAPPAGE__CONFIG", 		1);
 define("C__LDAPPAGE__SERVERTYPES",	2);
 define("C__LDAPPAGE__CONINFO",		3);
 
 define("C__LDAP_MAPPING__GROUP",			0);
 define("C__LDAP_MAPPING__OBJECT_CLASS",	1);
 define("C__LDAP_MAPPING__FIRSTNAME",		2);
 define("C__LDAP_MAPPING__LASTNAME",		3);
 define("C__LDAP_MAPPING__MAIL",			4);
 define("C__LDAP_MAPPING__USERNAME",		5);
 
 define("C__DEFAULT__TIMELIMIT",	30);
 
 class isys_module_ldap
 	extends isys_module
 {
 	private $m_userrequest;
 	
 	private $m_default_attributes = array("cn", "description");
 	
 	public static $m_ldapi;
 	
 	/**
 	* @param isys_module_request& $p_req
 	* @desc Initializes the module
 	*/
 	public function init(isys_module_request &$p_req)
 	{
 		if(is_object($p_req)) {
 		 $this->m_userrequest = &$p_req;
 		
 			return ISYS_TRUE;			
 		}
 		return ISYS_FALSE;
 	}
 	
 	/**
 	 * Resolves groups from ldap memberof array
 	 *
 	 * @param array $p_groups
 	 * @param isys_library_ldap $p_ldapi
 	 * @param isys_component_database $p_database
 	 */
 	public function ldap_get_groups($p_found_user, isys_component_database $p_database) {
 		$l_return = array();
 		
 		$l_dao = $this->get_dao($p_database);
 		
 		$l_ldap_groups 	= $p_found_user[C__LDAP_MAPPING__GROUP];
 		$l_ldapi 		= $p_found_user["ldapi"];
 		$l_server_type 	= $p_found_user["ldap_data"]["isys_ldap__isys_ldap_directory__id"];
 		
 		$l_server_data  = $l_dao->get_ldap_types($l_server_type)->__to_array();
 		
 		$l_mapping 		= isys_glob_unpack($l_server_data["isys_ldap_directory__mapping"]);
		$l_group_attr	= $l_mapping[C__LDAP_MAPPING__USERNAME];
 		
 		if (empty($l_group_id)) $l_group_id = "cn";
 		
 		if (is_array($l_ldap_groups) && is_object($l_ldapi)) {
 			
 			$l_dao_groups = new isys_contact_dao_group($p_database);
 			
 			foreach ($l_ldap_groups as $l_group) {
 				
 				$l_group_data		= $l_ldapi->read(	$l_group, 
 														"(objectclass=*)", 
 														array_merge(array($l_group_attr), 
 																	$this->m_default_attributes));
 				
	 			$l_group_entries	= $l_ldapi->get_entries($l_group_data);
 				
 				$l_idoit_group = $l_dao_groups->get_data("(isys_group__ldap_group = '".$l_group_entries[0][$l_group_attr][0]."')");
 				
 				if ($l_idoit_group->num_rows() > 0) {
	 				$l_idoit_group_data = $l_idoit_group->__to_array();
	 				
 					$l_return[] = $l_idoit_group_data;
 					
 				}
 				
 			}
 			
 		}
 		
 		return $l_return;
 	}
 	
 	/**
 	 * Returns a html list of ldap server types or fales, if no type was found.
 	 *
 	 * @return string / false
 	 */
 	public function get_server_types() {
 		global $g_comp_database;
 		
 		$l_ldap = new isys_module_dao_ldap($g_comp_database);
 		
 		$l_types = $l_ldap->get_ldap_types();
 		
 		if ($l_types->num_rows() > 0)
 		{
			$l_objList = new isys_component_list(ISYS_NULL, $l_types);
			$l_objList->createTempTable();
			
			$l_objList->config(
				array(
		 			"isys_ldap_directory__title" => "Title",
		 			"isys_ldap_directory__const" => "Const"
				),
				"?" . C__GET__MODULE_ID . "=" . C__MODULE__LDAP . "&".
				  C__GET__SETTINGS_PAGE."=".C__LDAPPAGE__SERVERTYPES . "&". 
				  C__CMDB__GET__EDITMODE."=".C__EDITMODE__ON.
				  "&id=[{isys_ldap_directory__id}]",
				"[{isys_ldap_directory__id}]",
				ISYS_TRUE);
			
			return $l_objList->getTempTableHtml();
 		} else return false;
 	}
 	
 	/**
 	 * Returns a html list of configured ldap servers, or false if nothing was configured, yet.
 	 *
 	 * @return string / false
 	 */
 	public function get_server_list() {
 		global $g_comp_database;
 		
 		$l_ldap = new isys_module_dao_ldap($g_comp_database);
 		
 		$l_data = $l_ldap->get_data();
 		
 		if ($l_data->num_rows() > 0)
 		{
			$l_objList = new isys_component_list(ISYS_NULL, $l_data);
			$l_objList->createTempTable();
			
			$l_objList->config(
				array(
					"isys_ldap_directory__title" => "Directory",
		 			"isys_ldap__hostname" => "Host",
		 			"isys_ldap__port" => "Port",
		 			"isys_ldap__dn" => "Login",
		 			"isys_ldap__user_search" => "Search",
		 			"isys_ldap__filter" => "Filter",
		 			"isys_ldap__active" => "Active",
				),
				"?" . C__GET__MODULE_ID . "=" . C__MODULE__LDAP . "&".
				  C__GET__SETTINGS_PAGE."=".C__LDAPPAGE__CONFIG . "&". 
				  C__CMDB__GET__EDITMODE."=".C__EDITMODE__ON.
				  "&id=[{isys_ldap__id}]",
				"[{isys_ldap__id}]",
				ISYS_TRUE);
			
			return $l_objList->getTempTableHtml();
 		} else return false;
 	}
 	
 	/**
 	 * Process ldap module
 	 *
 	 * @param array $p_get
 	 * @param array $p_post
 	 */
 	public function process($p_get, $p_post) {
 		global 	$g_comp_database, $g_comp_template, $g_dirs, 
 				$index_includes, $g_comp_template_navbar, $g_comp_template_language_manager;
 		
 		$l_tlm = &$g_comp_template_language_manager;
 		$l_ldap = new isys_module_dao_ldap($g_comp_database);
 		
 		/* Enable ajax button save */
 		$g_comp_template->assign("ajax_buttons", 	1);
 		$g_comp_template->assign("ajax_return", 	"ajax_return");
 		
 		switch ($p_get[C__GET__SETTINGS_PAGE]) {
 			case C__LDAPPAGE__SERVERTYPES:
 				
 				if ($_POST[C__GET__NAVMODE] == C__NAVMODE__SAVE) {
 					
 					$l_mapping = array(	C__LDAP_MAPPING__GROUP			=> $_POST["LDAP_MAP__GROUP"],
 										C__LDAP_MAPPING__OBJECT_CLASS 	=> $_POST["LDAP_MAP__OBJECTCLASS"],
 										C__LDAP_MAPPING__FIRSTNAME		=> $_POST["LDAP_MAP__GIVENNAME"],
 										C__LDAP_MAPPING__LASTNAME 		=> $_POST["LDAP_MAP__SURNAME"],
 										C__LDAP_MAPPING__MAIL 			=> $_POST["LDAP_MAP__MAIL"],
 										C__LDAP_MAPPING__USERNAME 		=> $_POST["LDAP_MAP__USERNAME"]);
 					
 					if (isset($_GET["id"])) {
 						
 						$l_ret = $l_ldap->save_ldap_directory(	$_POST["C__MODULE__LDAP_TYPE__TITLE"],
		 														$_POST["C__MODULE__LDAP_TYPE__CONST"],
		 														$l_mapping,
		 														$_GET["id"]);
	 						
 					} else {
 						
 						$l_ret = $l_ldap->create_ldap_directory(	$_POST["C__MODULE__LDAP_TYPE__TITLE"],
			 														$_POST["C__MODULE__LDAP_TYPE__CONST"],
			 														$l_mapping);
 						
 					}
 					
 					if ($l_ret) {
						$l_message = $l_tlm->{"LC__INFOBOX__DATA_WAS_SAVED"}."!";
					} else {
						$l_message = "Error! " . $l_tlm->{"LC__INFOBOX__DATA_WAS_NOT_SAVED"} . "<br />".
										$l_ldap->get_database_component()->get_last_error_as_string();
					}
 					
 					/* Die, because this is an ajax request */
 					die("<p class=\"note bold p5\">{$l_message}</p>");
 				}
 				
 				
 				if (isset($_GET["id"]) || $_POST[C__GET__NAVMODE] == C__NAVMODE__NEW) {
 					
 					/* Display single-view, if id is set */
 					if ($_GET["id"] > 0) {
 						
 						$l_types = $l_ldap->get_ldap_types($_GET["id"])->__to_array();
 						
 						$l_rules["C__MODULE__LDAP_TYPE__TITLE"]["p_strValue"]	= $l_types["isys_ldap_directory__title"];
 						
 						if ($l_types["isys_ldap_directory__const"] == "C__LDAP__AD" || 
 							$l_types["isys_ldap_directory__const"] == "C__LDAP__NDS" || 
 							$l_types["isys_ldap_directory__const"] == "C__LDAP__OPENLDAP") {
 							$l_rules["C__MODULE__LDAP_TYPE__CONST"]["p_bDisabled"] = true;
 						}
 						
 						$l_rules["C__MODULE__LDAP_TYPE__CONST"]["p_strValue"]	= $l_types["isys_ldap_directory__const"];
 						
 						$l_mapping = isys_glob_unpack($l_types["isys_ldap_directory__mapping"]);
 						$l_rules["LDAP_MAP__GROUP"]["p_strValue"]		= $l_mapping[C__LDAP_MAPPING__GROUP];
 						$l_rules["LDAP_MAP__OBJECTCLASS"]["p_strValue"]	= $l_mapping[C__LDAP_MAPPING__OBJECT_CLASS];
 						$l_rules["LDAP_MAP__GIVENNAME"]["p_strValue"]	= $l_mapping[C__LDAP_MAPPING__FIRSTNAME];
 						$l_rules["LDAP_MAP__SURNAME"]["p_strValue"]		= $l_mapping[C__LDAP_MAPPING__LASTNAME];
 						$l_rules["LDAP_MAP__MAIL"]["p_strValue"]		= $l_mapping[C__LDAP_MAPPING__MAIL];
 						$l_rules["LDAP_MAP__USERNAME"]["p_strValue"]	= $l_mapping[C__LDAP_MAPPING__USERNAME];
		 				
 					}
 					
 					$g_comp_template->smarty_tom_add_rules("tom.content.bottom.content", $l_rules);
 					
 				} else {
	 				$l_list = $this->get_server_types();
		 			$g_comp_template->assign("g_list", $l_list);
	 				
 				}
 				
 				$index_includes['contentbottomcontent'] = "./content/bottom/content/module__ldap_server_types.tpl";
 				
 				break;
 			default:
 			case C__LDAPPAGE__CONFIG:
 				
 				/* Connection test */
 				if ($_GET["connection_test"] == "1") {
 					
 					$l_ldapi = new isys_library_ldap(	$_POST["C__MODULE__LDAP__HOST"], 
 														$_POST["C__MODULE__LDAP__DN"],
 														$_POST["C__MODULE__LDAP__PASS"],
 														$_POST["C__MODULE__LDAP__PORT"],
 														$_POST["C__MODULE__LDAP__VERSION"],
 														!!$_POST["C__MODULE__LDAP__TLS"]);
 					
 					if (!$l_ldapi->connected()) {
	 					die("<p class=\"exception bold p5\">Error! Could not connect to ".
	 						"{$_POST["C__MODULE__LDAP__HOST"]}:{$_POST["C__MODULE__LDAP__PORT"]} using ".
	 						"{$_POST["C__MODULE__LDAP__DN"]}!<br />".
	 						"{$l_ldapi->get_last_error()}<br />".
	 						"LDAP-Error: {$l_ldapi->get_ldap_error()}</p>");
		 			} else {
		 				echo "<p class=\"note bold p5\">".
		 						"Connection OK!<br />";
		 				
		 				if ($_POST["C__MODULE__LDAP__RECURSIVE"] > 0) $l_scope = C__LDAP_SCOPE__RECURSIVE;
		 				else $l_scope = C__LDAP_SCOPE__SINGLE;
		 				
		 				$l_res = $l_ldapi->search($_POST["C__MODULE__LDAP__SEARCH"],
		 											$_POST["C__MODULE__LDAP__FILTER"],
		 											array(),
		 											0,
		 											50,
		 											(!empty($_POST["C__MODULE__LDAP__TIMELIMIT"])?
		 												intval($_POST["C__MODULE__LDAP__TIMELIMIT"]):
		 												C__DEFAULT__TIMELIMIT),
		 											NULL,
		 											$l_scope);
		 				
		 				if (($l_count = $l_ldapi->count($l_res))) {
		 					if ($l_count >= 50) {
		 						echo "<strong>50</strong> or more objects found in {$_POST["C__MODULE__LDAP__SEARCH"]}.";
		 					} else {
		 						echo "<strong>{$l_count}</strong> object(s) found in {$_POST["C__MODULE__LDAP__SEARCH"]}.";
		 					}
		 				} else {
		 					echo "No object found. That means that no one will be able to login with the current setup. ".
		 							"Check your filter and search-dn.";
		 				}
		 				
		 				echo "</p>";
		 				
		 				die();
		 			}
 				}
 				
 				/* Delete */
 				if ($_POST[C__GET__NAVMODE] == C__NAVMODE__PURGE) {
 					foreach ($_POST["id"] as $l_id) {
 						if ($l_ldap->delete_server($l_id)) {
 							// Server $l_id deleted successfully
 						}
 					}
 				}
 				
 				/* Save*/
 				if ($_POST[C__GET__NAVMODE] == C__NAVMODE__SAVE) {
 					
 					if (empty($_POST["C__MODULE__LDAP__ACTIVE"])) {
 						$_POST["C__MODULE__LDAP__ACTIVE"] = "0";
 					}
 					
 					try {
 						if ($_POST["C__MODULE__LDAP__PASS"] == $_POST["C__MODULE__LDAP__PASS_2"]) {
				 			if (isset($_GET["id"])) {
				 				
				 				$l_ret = $l_ldap->save_server(	$_POST["C__MODULE__LDAP__DIRECTORY"],
				 												$_POST["C__MODULE__LDAP__HOST"],
						 										$_POST["C__MODULE__LDAP__PORT"],
						 										$_POST["C__MODULE__LDAP__DN"],
						 										$_POST["C__MODULE__LDAP__PASS"],
						 										$_POST["C__MODULE__LDAP__SEARCH"],
						 										$_POST["C__MODULE__LDAP__FILTER"],
						 										$_POST["C__MODULE__LDAP__ACTIVE"],
						 										$_POST["C__MODULE__LDAP__TIMELIMIT"],
						 										$_POST["C__MODULE__LDAP__RECURSIVE"],
						 										$_POST["C__MODULE__LDAP__TLS"],
						 										$_POST["C__MODULE__LDAP__VERSION"],
						 										$_GET["id"]);
				 				
				 			} else {
				 				
					 				$l_ret = $l_ldap->create_server(	$_POST["C__MODULE__LDAP__DIRECTORY"],
					 													$_POST["C__MODULE__LDAP__HOST"],
								 										$_POST["C__MODULE__LDAP__PORT"],
								 										$_POST["C__MODULE__LDAP__DN"],
								 										$_POST["C__MODULE__LDAP__PASS"],
								 										$_POST["C__MODULE__LDAP__SEARCH"],
								 										$_POST["C__MODULE__LDAP__FILTER"],
								 										$_POST["C__MODULE__LDAP__ACTIVE"],
								 										$_POST["C__MODULE__LDAP__TIMELIMIT"],
								 										$_POST["C__MODULE__LDAP__RECURSIVE"],
								 										$_POST["C__MODULE__LDAP__TLS"],
								 										$_POST["C__MODULE__LDAP__VERSION"]);
				 				
				 			}
 						} else {
 							$l_message = "Passwords does not match!";
 						}
 					} catch (Exception $e) {
	 					$l_message = $e->getMessage();
	 				}
	 				
		 			/* Die, because this is an ajax request */
		 			if (!$l_ret) {
	 					die("<p class=\"exception bold p5\">Error! Could not save!<br />{$l_message}</p>");
		 			} else {
		 				die("<p class=\"note bold p5\">".$l_tlm->{"LC__INFOBOX__DATA_WAS_SAVED"}."!</p>");
		 			}
 				}
 				
 				/* New or Edit */
 				if (isset($_GET["id"]) || $_POST[C__GET__NAVMODE] == C__NAVMODE__NEW) {
 					/* Single-View Mode */
 				
 					if ($_GET[C__CMDB__GET__EDITMODE] == C__EDITMODE__OFF || !isset($_GET[C__CMDB__GET__EDITMODE])) {
 						$g_comp_template_navbar->set_active(ISYS_TRUE, C__NAVBAR_BUTTON__EDIT);
 					}
 					
 					$l_rules["C__MODULE__LDAP__ACTIVE"]["p_arData"] = isys_glob_pack(get_smarty_arr_YES_NO());
 					$l_rules["C__MODULE__LDAP__PORT"]["p_strValue"]	= "389";
 					$l_rules["C__MODULE__LDAP__TIMELIMIT"]["p_strValue"]	= "30";
 					$l_rules["C__MODULE__LDAP__FILTER"]["p_strValue"]	= "(objectClass=user)";
 					
 					$l_versions = array("1" => "1", "2" => "2", "3" => "3");
 					$l_rules["C__MODULE__LDAP__VERSION"]["p_arData"] = isys_glob_pack($l_versions);
 					$l_rules["C__MODULE__LDAP__VERSION"]["p_strSelectedID"] = "3";
 					
 					$l_rules["C__MODULE__LDAP__TLS"]["p_arData"] = isys_glob_pack(get_smarty_arr_YES_NO());
 					
 					if ($_GET["id"] > 0) {
	 					$l_data = $l_ldap->get_data($_GET["id"])->__to_array();
	 					
	 					$g_comp_template->assign("g_recursive", $l_data["isys_ldap__recursive"]);
	 					
	 					$l_rules["C__MODULE__LDAP__TLS"]["p_strSelectedID"] = $l_data["isys_ldap__tls"];
	 					$l_rules["C__MODULE__LDAP__VERSION"]["p_strSelectedID"] = $l_data["isys_ldap__version"];
	 					
	 					$l_rules["C__MODULE__LDAP__ACTIVE"]["p_strSelectedID"] = $l_data["isys_ldap__active"];
		 				$l_rules["C__MODULE__LDAP__DIRECTORY"]["p_strSelectedID"] = $l_data["isys_ldap__isys_ldap_directory__id"];
	 					
		 				$l_rules["C__MODULE__LDAP__TITLE"]["p_strValue"]	= $l_data["isys_ldap__title"];
		 				
		 				$l_rules["C__MODULE__LDAP__TIMELIMIT"]["p_strValue"]	= $l_data["isys_ldap__timelimit"];
		 				
		 				$l_rules["C__MODULE__LDAP__HOST"]["p_strValue"]	= $l_data["isys_ldap__hostname"];
		 				$l_rules["C__MODULE__LDAP__PORT"]["p_strValue"]	= $l_data["isys_ldap__port"];
		 				
		 				$l_rules["C__MODULE__LDAP__DN"]["p_strValue"]	= $l_data["isys_ldap__dn"];
		 				$l_rules["C__MODULE__LDAP__PASS"]["p_strValue"]	= $l_data["isys_ldap__password"];
		 				$l_rules["C__MODULE__LDAP__PASS_2"]["p_strValue"]	= $l_data["isys_ldap__password"];
		 				
		 				$l_rules["C__MODULE__LDAP__SEARCH"]["p_strValue"]	= $l_data["isys_ldap__user_search"];
		 				$l_rules["C__MODULE__LDAP__FILTER"]["p_strValue"]	= $l_data["isys_ldap__filter"];
		 				
 					}
 					
 					$g_comp_template->smarty_tom_add_rules("tom.content.bottom.content", $l_rules);
 					
 				} else {
 					/* List-Mode */
 					
 					$g_comp_template_navbar->set_active(ISYS_TRUE, C__NAVBAR_BUTTON__PURGE);
 					$g_comp_template_navbar->set_active(ISYS_TRUE, C__NAVBAR_BUTTON__NEW);
 					
	 				$l_list = $this->get_server_list();
	 				$g_comp_template->assign("g_list", $l_list);
	 				
 				}
 				
 				$index_includes['contentbottomcontent'] = "./content/bottom/content/module__ldap.tpl";
 				break;
 		}
 		
 	}
 	
 	
 	/**
 	* @desc Starts module process
 	* @throws isys_exception_general
 	*/
 	public function start()
 	{
 		/* Unpack request package ;-) */
 		$l_gets 		= $this->m_userrequest->get_gets(); 		
 		$l_posts    	= $this->m_userrequest->get_posts();
 		$l_template 	= $this->m_userrequest->get_template();
 		$l_tree     	= $this->m_userrequest->get_menutree();
 		//$l_objNavbar	= $this->m_userrequest->get_navbar();
 		   
 		/* Don't forget to create the root entry */
 		$l_tree->add_node(0, -1, "LDAP");
 		
 		if (!isset($_GET[C__GET__SETTINGS_PAGE])) $_GET[C__GET__SETTINGS_PAGE] = C__LDAPPAGE__CONFIG;
 		
 		/* And now the other nodes ... */
 		$l_tree->add_node(1, 0, "Server", 
 								"?" . C__GET__MODULE_ID . "=" . C__MODULE__LDAP . "&".
	 								C__GET__SETTINGS_PAGE."=".C__LDAPPAGE__CONFIG . "&". 
	 								C__CMDB__GET__EDITMODE."=".C__EDITMODE__ON,
	 							NULL, "images/icons/tree/stammdaten.gif", ($_GET[C__GET__SETTINGS_PAGE]==C__LDAPPAGE__CONFIG)?1:0);
 		
	 	$l_tree->add_node(2, 0, "Directories", 
 								"?" . C__GET__MODULE_ID . "=" . C__MODULE__LDAP . "&".
	 								C__GET__SETTINGS_PAGE."=".C__LDAPPAGE__SERVERTYPES,
	 							NULL, "images/icons/tree/tapelibrary.gif", ($_GET[C__GET__SETTINGS_PAGE]==C__LDAPPAGE__SERVERTYPES)?1:0);
	 							
	 	 /* Assign tree */
		$l_template->assign("menu_tree", $l_tree->process(0));
	 	
		$this->process($l_gets, $l_posts);
		
 	}
 	
 	/**
 	 * Returns dao instnace
 	 *
 	 * @param isys_component_database $p_database
 	 * @return isys_module_dao_ldap
 	 */
 	public function get_dao(&$p_database = NULL) {
 		global $g_comp_database;
 		
 		if (is_null($p_database)) $l_database = $g_comp_database;
 		else $l_database = $p_database;
 		
 		return new isys_module_dao_ldap($l_database);
 	}
 	
 	/**
 	 * Returns ldap library by isys_ldap__id
 	 *
 	 * @param int $p_ldap_id
 	 * 
 	 * @return isys_library_ldap
 	 */
 	public function get_library_by_id($p_ldap_id, &$p_connection_info) {
 		
 		/* get isys_module_dao_ldap */
 		$l_dao = $this->get_dao();
 		
 		/* retrieve server configuration data */
 		$l_ldap = $l_dao->get_active_servers($p_ldap_id)->__to_array();
 		
 		/* fill reference with ldap connection information */
 		$p_connection_info = $l_ldap;
 		
 		try {
	 		/* return library */
	 		return $this->get_library(	$l_ldap["isys_ldap__hostname"],
										$l_ldap["isys_ldap__dn"],
										$l_ldap["isys_ldap__password"],
										$l_ldap["isys_ldap__port"],
										$l_ldap["isys_ldap__version"],
										$l_ldap["isys_ldap__tls"]);
 		} catch (Exception $e) {
 			throw $e;
 		}
 		
 	}
 	
 	/**
 	 * Returns internal ldap library
 	 *
 	 * @param string $p_host
 	 * @param string $p_user_dn
 	 * @param string $p_pass
 	 * @param int [$p_port]
 	 * 
 	 * @return isys_library_ldap
 	 */
 	public function get_library($p_host, $p_user_dn, $p_pass, $p_port = 389, $p_protocol_version = 3, $p_tls = false) {
 		if ($p_port < 0 || empty($p_port)) $p_port = 389;
 		if ($p_protocol_version < 1 || empty($p_protocol_version)) $p_protocol_version = 3;
 		if (empty($tls)) $p_tls = false;
 		
 		try {
	 		return new isys_library_ldap(	$p_host, 
											$p_user_dn,
											$p_pass,
											$p_port,
											$p_protocol_version,
											$p_tls);
 		} catch (Exception $e) {
 			throw $e;
 		}
 	}
 	
 	/**
 	* @desc Starts XML process
 	*/
 	public function xmlproc()
 	{
 		
 	}
 	
 	
 	/**
 	* @desc Constructor
 	*/
 	public function __construct() {
 		parent::__construct($this);
 	}
 }
?>